@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2012 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2012-2015 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,14 +24,18 @@
 #include "alsathread.h"
 
 
-Jackclient::Jackclient (const char *jname, const char*jserv, int mode, int nchan) :
+Jackclient::Jackclient (const char *jname, const char *jserv, int mode, int nchan, bool sync) :
     _client (0),
     _mode (mode),
     _nchan (nchan),     
     _state (INIT),
-    _freew (false)
+    _freew (false),
+    _resamp (0)
 {
     init (jname, jserv);
+    if (!sync) _resamp = new VResampler ();
+    // Time wraps around after 2^28 usecs.
+    _jtmod = ldexp (1e-6, 28);
 }
 
 
@@ -101,6 +105,7 @@ void Jackclient::fini (void)
         jack_client_close (_client);
     }
     delete[] _buff;
+    delete _resamp;
 }
 
 
@@ -153,14 +158,17 @@ void Jackclient::start (Lfq_audio   *audioq,
     _commq = commq;
     _alsaq = alsaq;
     _infoq = infoq;
-    _quant = ldexp (1e-6f, 28);
     _ratio = ratio;
+    _delay = delay;
     _rcorr = 1.0;
-    _resamp.setup (_ratio, _nchan, rqual);
-    _resamp.set_rrfilt (100);
-    d = _resamp.inpsize () / 2.0;
-    if (_mode == PLAY) d *= _ratio;
-    _delay = delay + d;
+    if (_resamp)
+    {
+        _resamp->setup (_ratio, _nchan, rqual);
+        _resamp->set_rrfilt (100);
+        d = _resamp->inpsize () / 2.0;
+        if (_mode == PLAY) d *= _ratio;
+        _delay += d;
+    }
     _ltcor = ltcor;
     _ppsec = (_fsamp + _bsize / 2) / _bsize;
     initwait (_ppsec / 2);
@@ -183,15 +191,17 @@ void Jackclient::initsync (void)
     _commq->reset ();
     _alsaq->reset ();
     _audioq->reset ();
-    // Reset and prefill the resampler.
-    _resamp.reset ();
-    _resamp.inp_count = _resamp.inpsize () / 2 - 1;
-    _resamp.out_count = 10000;
-    _resamp.process ();
+    if (_resamp)
+    {
+        // Reset and prefill the resampler.
+	_resamp->reset ();
+	_resamp->inp_count = _resamp->inpsize () / 2 - 1;
+	_resamp->out_count = 99999;
+	_resamp->process ();
+    }
     // Initiliase state variables.
     _t_a0 = _t_a1 = 0;
     _k_a0 = _k_a1 = 0;
-    _k_j0 = 0;
     // Initialise loop filter state.
     _z1 = _z2 = _z3 = 0;
     // Activate the ALSA thread,
@@ -218,28 +228,55 @@ void Jackclient::playback (int nframes)
 {
     int    i, j, n;
     float  *p, *q;
+    float  *inp [MAXCHAN];
 
-    // Interleave inputs into _buff.
+    _bstat = _audioq->rd_avail ();
     for (i = 0; i < _nchan; i++)
     {
-	p = (float *)(jack_port_get_buffer (_ports [i], nframes));
-	q = _buff + i;
-	for (j = 0; j < _bsize; j++) q [j * _nchan] = p [j];
-    }       
-    // Resample _buff and write to audio queue.
-    // The while loop takes care of wraparound.
-    _resamp.inp_count = _bsize;
-    _resamp.inp_data  = _buff;
-    while (_resamp.inp_count)
+        inp [i] = (float *)(jack_port_get_buffer (_ports [i], nframes));
+    }
+    if (_resamp)
+    {    
+	// Interleave inputs into _buff.
+	for (i = 0; i < _nchan; i++)
+	{
+	    p = inp [i];
+	    q = _buff + i;
+	    for (j = 0; j < _bsize; j++) q [j * _nchan] = p [j];
+	}       
+	// Resample _buff and write to audio queue.
+	// The while loop takes care of wraparound.
+	_resamp->inp_count = _bsize;
+	_resamp->inp_data  = _buff;
+	while (_resamp->inp_count)
+	{
+	    _resamp->out_count = _audioq->wr_linav ();
+	    _resamp->out_data  = _audioq->wr_datap ();
+	    n = _resamp->out_count;
+	    _resamp->process ();
+	    n -= _resamp->out_count;
+	    _audioq->wr_commit (n);
+	}
+    }
+    else
     {
-	_resamp.out_count = _audioq->wr_linav ();
-	_resamp.out_data  = _audioq->wr_datap ();
-	n = _resamp.out_count;
-	_resamp.process ();
-	n -= _resamp.out_count;
-	_audioq->wr_commit (n);
-	// Adjust state by the number of frames used.
-	_k_j0 += n;
+        // Interleave inputs into audio queue.
+	// The while loop takes care of wraparound.
+	while (nframes)
+	{
+	    q = _audioq->wr_datap ();
+	    n = _audioq->wr_linav ();
+	    if (n > nframes) n = nframes;
+	    for (i = 0; i < _nchan; i++)
+	    {
+	        p = inp [i];
+	        for (j = 0; j < n; j++) q [j * _nchan] = p [j];
+		inp [i] += n;
+		q += 1;
+	    }
+	    _audioq->wr_commit (n);
+	    nframes -= n;
+	}
     }
 }
 
@@ -248,29 +285,56 @@ void Jackclient::capture (int nframes)
 {
     int    i, j, n;
     float  *p, *q;
+    float  *out [MAXCHAN];
 
-    // Read from audio queue and resample.
-    // The while loop takes care of wraparound.
-    _resamp.out_count = _bsize;
-    _resamp.out_data  = _buff;
-    while (_resamp.out_count)
-    {
-	_resamp.inp_count = _audioq->rd_linav ();
-	_resamp.inp_data  = _audioq->rd_datap ();
-	n = _resamp.inp_count;
-	_resamp.process ();
-	n -= _resamp.inp_count;
-	_audioq->rd_commit (n);
-	// Adjust state by the number of frames used.
-	_k_j0 += n;
-    }
-    // Deinterleave _buff to outputs.
     for (i = 0; i < _nchan; i++)
     {
-	p = _buff + i;
-	q = (float *)(jack_port_get_buffer (_ports [i], nframes));
-	for (j = 0; j < _bsize; j++) q [j] = p [j * _nchan];
-    }       
+        out [i] = (float *)(jack_port_get_buffer (_ports [i], nframes));
+    }
+    if (_resamp)
+    {
+	// Read from audio queue and resample.
+	// The while loop takes care of wraparound.
+	_resamp->out_count = _bsize;
+	_resamp->out_data  = _buff;
+	while (_resamp->out_count)
+	{
+	    _resamp->inp_count = _audioq->rd_linav ();
+	    _resamp->inp_data  = _audioq->rd_datap ();
+	    n = _resamp->inp_count;
+	    _resamp->process ();
+	    n -= _resamp->inp_count;
+	    _audioq->rd_commit (n);
+	}
+	// Deinterleave _buff to outputs.
+	for (i = 0; i < _nchan; i++)
+	{
+	    p = _buff + i;
+	    q = out [i];
+	    for (j = 0; j < _bsize; j++) q [j] = p [j * _nchan];
+	}
+    }
+    else
+    {
+        // Deinterleave audio queue to outputs.
+	// The while loop takes care of wraparound.
+	while (nframes)
+	{
+	    p = _audioq->rd_datap ();
+	    n = _audioq->rd_linav ();
+	    if (n > nframes) n = nframes;
+	    for (i = 0; i < _nchan; i++)
+	    {
+	        q = out [i];
+	        for (j = 0; j < n; j++) q [j] = p [j * _nchan];
+		out [i] += n;
+		p += 1;
+	    }
+	    _audioq->rd_commit (n);
+	    nframes -= n;
+	}
+    }
+    _bstat = _audioq->rd_avail ();
 }
 
 
@@ -298,6 +362,7 @@ void Jackclient::sendinfo (int state, double error, double ratio)
 	J->_state = state;
 	J->_error = error;
 	J->_ratio = ratio;
+	J->_bstat = _bstat;
 	_infoq->wr_commit ();
     }
 }
@@ -337,8 +402,10 @@ int Jackclient::jack_process (int nframes)
 {
     int             dk, n;
     Adata           *D;
+    jack_time_t     t0, t1;
     jack_nframes_t  ft;
-    double          tj, err, d1, d2;
+    float           us;
+    double          tj, err, d1, d2, rd;
 
     // Buffer size change or other evil.
     if (_state == TERM)
@@ -361,16 +428,27 @@ int Jackclient::jack_process (int nframes)
         else return 0;
     }
 
-    // Compute the start time of the current cycle.
-    // Jack should really provide the usecs directly.
-    ft = jack_last_frame_time (_client);
-    tj = 1e-6 * (jack_frames_to_time (_client, ft) & 0x0FFFFFFF);
-
-    // Check for any skipped cycles. This is invalid
-    // the first time, but won't be used then anyway.
-    dk = ft - _ft - _bsize;
+    // Get the start time of the current cycle.
+    jack_get_cycle_times (_client, &ft, &t0, &t1, &us);
+    tj = jacktime (t0);
+    
+    // Check for any skipped cycles.
+    if (_state >= SYNC1)
+    {
+        dk = ft - _ft - _bsize;
+        if (_mode == PLAY)
+	{
+	    dk = (int)(dk * _ratio + 0.5);
+            _audioq->wr_commit (dk);
+	}
+	else
+	{
+	    dk = (int)(dk / _ratio + 0.5);    
+            _audioq->rd_commit (dk);
+	}
+    }
     _ft = ft;
-
+    
     // Check if we have timing data from the ALSA thread.
     n = _alsaq->rd_avail ();
     // If the data queue is full restart synchronisation.
@@ -410,33 +488,27 @@ int Jackclient::jack_process (int nframes)
         // Compute the delay error.
         d1 = modtime (tj - _t_a0);
         d2 = modtime (_t_a1 - _t_a0);
+	rd = _resamp ? _resamp->inpdist () : 0.0;
+	
 	if (_mode == PLAY)
 	{
-            n = _k_j0 - _k_a0; // Must be done as integer as both terms will overflow.
-            err = n - (_k_a1 - _k_a0) * d1 / d2  + _resamp.inpdist () * _ratio - _delay;
+            n = _audioq->nwr () - _k_a0; // Must be done as integer as both terms will overflow.
+            err = n - (_k_a1 - _k_a0) * d1 / d2  + rd * _ratio - _delay;
 	}
 	else
 	{
-            n = _k_a0 - _k_j0;
-            err = n + (_k_a1 - _k_a0) * d1 / d2  + _resamp.inpdist () - _delay ;
+            n = _k_a0 - _audioq->nrd (); // Must be done as integer as both terms will overflow.
+            err = n + (_k_a1 - _k_a0) * d1 / d2  + rd - _delay ;
 	}
 
         n = (int)(floor (err + 0.5));
         if (_state == SYNC2)
         {
-            // We have the first delay error value. Adjust both the state
-            // variables and the audio queue by the same number of frames
-            // to obtain the actually wanted delay, and start tracking.
-	    if (_mode == PLAY)
-	    {
-                _audioq->wr_commit (-n);
-                _k_j0 -= n;
-	    }
-	    else
-	    {
-                _audioq->rd_commit (n);
-                _k_j0 += n;
-	    }
+            // We have the first delay error value. Adjust the audio
+	    // queue to obtain the actually wanted delay, and start
+	    // tracking.
+	    if (_mode == PLAY) _audioq->wr_commit (-n);
+	    else               _audioq->rd_commit (n);
             err -= n;
             setloop (1.0);
             _state = PROC1;
@@ -444,39 +516,10 @@ int Jackclient::jack_process (int nframes)
         else if (_state >= PROC1)
         {
 	    // Check error conditions.
-	    if (dk)
-	    {
-                // Jack skipped some cycles. Adjust both the state
-                // and the audio queue so we can just continue.
-		// The loop will correct the remaining fraction
-		// of a frame.
-	        if (_mode == PLAY)
-		{
-		    dk = (int)(dk * _ratio + 0.5);
-                    if (abs (dk + n) < _bsize / 4)
-                    {
-                        _audioq->wr_commit (dk);
-                        _k_j0 += dk;
-                        err += dk;
-		        n = 0;
-		    }
-		}
-		else
-		{
-		    dk = (int)(dk / _ratio + 0.5);    
-                    if (abs (dk - n) < _bsize / 4)
-		    {                
-                        _audioq->rd_commit (dk);
-                        _k_j0 += dk;
-                        err -= dk;
-		        n = 0;
-		    }
-		}
-            }
-            if (fabs (err) >= 50)
+            if (fabs (err) >= 100)
             {
-                // Something is really wrong, wait 15 seconds then restart.
-                initwait (15 * _ppsec);
+                // Something is really wrong, wait 10 seconds then restart.
+                initwait (10 * _ppsec);
 		return 0;
             }
         }
@@ -492,13 +535,16 @@ int Jackclient::jack_process (int nframes)
     if (_state >= PROC1)
     {
         // Run loop filter and set resample ratio.
-        _z1 += _w0 * (_w1 * err - _z1);
-        _z2 += _w0 * (_z1 - _z2);
-        _z3 += _w2 * _z2;
-        _rcorr = 1 - _z2 - _z3;
-	if (_rcorr > 1.05) _rcorr = 1.05;
-	if (_rcorr < 0.95) _rcorr = 0.95;
-        _resamp.set_rratio (_rcorr);
+	if (_resamp)
+	{
+            _z1 += _w0 * (_w1 * err - _z1);
+            _z2 += _w0 * (_z1 - _z2);
+            _z3 += _w2 * _z2;
+            _rcorr = 1 - _z2 - _z3;
+	    if (_rcorr > 1.05) _rcorr = 1.05;
+	    if (_rcorr < 0.95) _rcorr = 0.95;
+            _resamp->set_rratio (_rcorr);
+	}
 	sendinfo (_state, err, _rcorr);
 
 	// Resample and transfer between audio
