@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2012 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2012-2015 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -31,10 +31,10 @@ class Jackclient
 {
 public:
 
-    Jackclient (const char  *jname, const char *jserv, int mode, int nchan);
+    Jackclient (const char  *jname, const char *jserv, int mode, int nchan, bool sync);
     virtual ~Jackclient (void);
     
-    enum { PLAY, CAPT };
+    enum { PLAY, CAPT, MAXCHAN = 64 };
     enum { INIT, TERM, WAIT, SYNC0, SYNC1, SYNC2, PROC1, PROC2 };
 
     void start (Lfq_audio   *audioq,
@@ -58,11 +58,16 @@ private:
 
     double modtime (double d)
     {
-	if (d < -200) d += _quant;
-	if (d >  200) d -= _quant;
+	if (d < -150.0) d += _jtmod;
+	if (d >  150.0) d -= _jtmod;
 	return d;
     }
 
+    double jacktime (jack_time_t t)
+    {
+        return 1e-6 * (t & 0x0FFFFFFFLL);
+    }
+    
     void initwait (int nwait);
     void initsync (void);
     void setloop (double bw);
@@ -78,7 +83,7 @@ private:
     int  jack_process (int nframes);
 
     jack_client_t  *_client;
-    jack_port_t    *_ports [64];
+    jack_port_t    *_ports [MAXCHAN];
     const char     *_jname;
     int             _mode;
     int             _nchan;
@@ -95,15 +100,15 @@ private:
     Lfq_adata      *_alsaq;
     Lfq_jdata      *_infoq;
     double          _ratio;
-    double          _quant;
+    double          _jtmod;
     int             _ppsec;
+    int             _bstat;
 
     jack_nframes_t  _ft;
     double          _t_a0;
     double          _t_a1;
     int             _k_a0;
     int             _k_a1;
-    int             _k_j0;
     double          _delay;
     int             _ltcor;
 
@@ -114,7 +119,7 @@ private:
     double           _z2;
     double           _z3;
     double          _rcorr;
-    VResampler      _resamp;
+    VResampler     *_resamp;
 
     static void jack_static_shutdown (void *arg);
     static int  jack_static_buffsize (jack_nframes_t nframes, void *arg);
