@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2012 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2012-2015 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -34,24 +34,25 @@ static Lfq_jdata   infoq (256);
 static Lfq_audio  *audioq = 0;
 static bool stop = false;
 
-static const char   *clopt = "hvLj:d:r:p:n:c:Q:I:";
+static const char   *clopt = "hvLSj:d:r:p:n:c:Q:I:";
 static bool          v_opt = false;
 static bool          L_opt = false;
+static bool          S_opt = false;
 static const char   *jname = APPNAME;
 static const char   *device = 0;
 static int           fsamp = 48000;
 static int           bsize = 256;
 static int           nfrag = 2;
 static int           nchan = 2;
-static int           rqual = 48;
+static int           rqual = 0;
 static int           ltcor = 0;
 
 
 static void help (void)
 {
     fprintf (stderr, "\n%s-%s\n", APPNAME, VERSION);
-    fprintf (stderr, "(C) 2012-2013 Fons Adriaensen  <fons@linuxaudio.org>\n");
-    fprintf (stderr, "Use ALSA capture device as a Jack client, with resampling.\n\n");
+    fprintf (stderr, "(C) 2012-2015 Fons Adriaensen  <fons@linuxaudio.org>\n");
+    fprintf (stderr, "Use ALSA capture device as a Jack client.\n\n");
     fprintf (stderr, "Usage: %s <options>\n", APPNAME);
     fprintf (stderr, "Options:\n");
     fprintf (stderr, "  -h                 Display this text\n");
@@ -61,8 +62,9 @@ static void help (void)
     fprintf (stderr, "  -p <period>        Period size [256]\n");   
     fprintf (stderr, "  -n <nfrags>        Number of fragments [2]\n");   
     fprintf (stderr, "  -c <nchannels>     Number of channels [2]\n");
-    fprintf (stderr, "  -Q <quality>       Resampling quality [48]\n");
-    fprintf (stderr, "  -I <latency>       Latency adjustment[0]\n");
+    fprintf (stderr, ", -S                 Word clock sync, no resampling\n");
+    fprintf (stderr, "  -Q <quality>       Resampling quality, 16..96 [auto]\n");
+    fprintf (stderr, "  -I <samples>       Latency adjustment [0]\n");
     fprintf (stderr, "  -L                 Force 16-bit and 2 channels [off]\n");
     fprintf (stderr, "  -v                 Print tracing information [off]\n");
     exit (1);
@@ -88,6 +90,7 @@ static void procoptions (int ac, char *av [])
         case 'h' : help (); exit (0);
         case 'v' : v_opt = true; break;
         case 'L' : L_opt = true; break;
+	case 'S' : S_opt = true; break;
         case 'j' : jname = optarg; break;
         case 'd' : device = optarg; break;
 	case 'r' : fsamp = atoi (optarg); break;    
@@ -127,11 +130,12 @@ static void sigint_handler (int)
 
 static void printinfo (void)
 {
-    int     n;
+    int     n, k;
     double  e, r;
     Jdata   *J;
 
     n = 0;
+    k = 99999;
     e = r = 0;
     while (infoq.rd_avail ())
     {
@@ -144,8 +148,7 @@ static void printinfo (void)
 	}
 	else if (J->_state == Jackclient::WAIT)
 	{
-	    printf ("Detected excessive timing errors, waiting 15 seconds.\n");
-	    printf ("This may happen with current Jack1 after freewheeling.\n");
+	    printf ("Detected excessive timing errors, waiting 10 seconds.\n");
 	    n = 0;
 	}
 	else if (J->_state == Jackclient::SYNC0)
@@ -157,10 +160,11 @@ static void printinfo (void)
 	    n++;
 	    e += J->_error;
 	    r += J->_ratio;
+	    if (J->_bstat < k) k = J->_bstat;
 	}
 	infoq.rd_commit ();
     }
-    if (n) printf ("%8.3lf %10.6lf\n", e / n, r / n);
+    if (n) printf ("%8.3lf %10.6lf %5d\n", e / n, r / n, k);
 }
 
 
@@ -200,17 +204,25 @@ int main (int ac, char *av [])
 	fprintf (stderr, "Warning: only %d channels are available.\n", nchan);
     }
     C = new Alsathread (A, Alsathread::CAPT);
-    J = new Jackclient (jname, 0, Jackclient::CAPT, nchan);
+    J = new Jackclient (jname, 0, Jackclient::CAPT, nchan, S_opt);
     usleep (100000);
 
     t_alsa = (double) bsize / fsamp;
     if (t_alsa < 1e-3) t_alsa = 1e-3;
     t_jack = (double) J->bsize () / J->fsamp (); 
-    t_del = 1.5 * t_alsa + t_jack;
+    t_del = t_alsa + t_jack;
     k_del = (int)(t_del * fsamp);
     for (k = 256; k < k_del + J->bsize (); k *= 2);
     audioq = new Lfq_audio (k, nchan);
 
+    if (rqual == 0)
+    {
+        k = (fsamp < J->fsamp ()) ? fsamp : J->fsamp ();
+        if (k < 44100) k = 44100;
+        rqual = (int)((6.7 * k) / (k - 38000));
+        if (rqual < 16) rqual = 16;
+    }
+
     C->start (audioq, &commq, &alsaq, J->rprio () + 10);
     J->start (audioq, &commq, &alsaq, &infoq, J->fsamp () / (double) fsamp, k_del, ltcor, rqual);
 
